@@ -1,5 +1,6 @@
 #!/usr/local/bin/lua5.1
 -- See Copyright Notice in license.html
+-- $Id: test.lua,v 1.52 2008/06/30 10:43:03 blumf Exp $
 
 TOTAL_FIELDS = 40
 TOTAL_ROWS = 40 --unused
@@ -10,6 +11,33 @@ QUERYING_STRING_TYPE_NAME = "text"
 CREATE_TABLE_RETURN_VALUE = 0
 DROP_TABLE_RETURN_VALUE = 0
 
+MSG_CURSOR_NOT_CLOSED = "cursor was not automatically closed by fetch"
+
+CHECK_GETCOL_INFO_TABLES = true
+
+---------------------------------------------------------------------
+if not string.find(_VERSION, " 5.0") then
+	table.getn = assert((loadstring or load)[[return function (t) return #t end]])()
+end
+
+---------------------------------------------------------------------
+-- Creates a table that can handle differing capitalization of field
+-- names
+-- @return A table with altered metatable
+---------------------------------------------------------------------
+local mt = {
+	__index = function(t, i)
+		if type(i) == "string" then
+			return rawget(t, string.upper(i)) or rawget(t, string.lower(i))
+		end
+
+		return rawget(t, i)
+	end
+}
+function fetch_table ()
+	return setmetatable({}, mt)
+end
+
 ---------------------------------------------------------------------
 -- Produces a SQL statement which completely erases a table.
 -- @param table_name String with the name of the table.
@@ -33,12 +61,30 @@ function assert2 (expected, value, msg)
 		tostring(expected)..")")
 end
 
+---------------------------------------------------------------------
+-- Shallow compare of two tables
+---------------------------------------------------------------------
+function table_compare(t1, t2)
+	if t1 == t2 then return true; end
+
+	for i, v in pairs(t1) do
+		if t2[i] ~= v then return false; end
+	end
+
+	for i, v in pairs(t2) do
+		if t1[i] ~= v then return false; end
+	end
+
+	return true
+end
+
 ---------------------------------------------------------------------
 -- object test.
 ---------------------------------------------------------------------
 function test_object (obj, objmethods)
 	-- checking object type.
 	assert2 (true, type(obj) == "userdata" or type(obj) == "table", "incorrect object type")
+
 	-- trying to get metatable.
 	assert2 ("LuaSQL: you're not allowed to get this metatable",
 		getmetatable(obj), "error permitting access to object's metatable")
@@ -91,7 +137,7 @@ function basic_test ()
 	assert2 (true, conn:close(), "couldn't close connection")
 	-- trying to execute a statement with a closed connection.
 	assert2 (false, pcall (conn.execute, conn, "create table x (c char)"),
-		"error connecting with a closed environment")
+		"error while executing through a closed connection")
 	-- it is ok to close a closed object, but false is returned instead of true.
 	assert2 (false, conn:close())
 	-- Check error situation.
@@ -145,7 +191,7 @@ function fetch2 ()
 	assert2 ('c', f2)
 	assert2 (nil, f3)
 	assert2 (nil, cur:fetch())
-	assert2 (true, cur:close(), "couldn't close cursor")
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	assert2 (false, cur:close())
 	-- insert a second record.
 	assert2 (1, CONN:execute ("insert into t (f1, f2) values ('d', 'e')"))
@@ -159,7 +205,7 @@ function fetch2 ()
 	assert2 ('e', f2)
 	assert2 (nil, f3)
 	assert2 (nil, cur:fetch())
-	assert2 (true, cur:close(), "couldn't close cursor")
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	assert2 (false, cur:close())
 	-- remove records.
 	assert2 (2, CONN:execute ("delete from t where f1 in ('b', 'd')"))
@@ -175,7 +221,7 @@ function fetch_new_table ()
 	assert2 (1, CONN:execute ("insert into t (f1, f2, f3, f4) values ('f', 'g', 'h', 'i')"))
 	-- retrieve data using a new table.
 	local cur = CUR_OK (CONN:execute ("select f1, f2, f3, f4 from t order by f1"))
-	local row, err = cur:fetch{}
+	local row, err = cur:fetch(fetch_table())
 	assert2 (type(row), "table", err)
 	assert2 ('a', row[1])
 	assert2 ('b', row[2])
@@ -185,7 +231,7 @@ function fetch_new_table ()
 	assert2 (nil, row.f2)
 	assert2 (nil, row.f3)
 	assert2 (nil, row.f4)
-	row, err = cur:fetch{}
+	row, err = cur:fetch(fetch_table())
 	assert (type(row), "table", err)
 	assert2 ('f', row[1])
 	assert2 ('g', row[2])
@@ -195,14 +241,14 @@ function fetch_new_table ()
 	assert2 (nil, row.f2)
 	assert2 (nil, row.f3)
 	assert2 (nil, row.f4)
-	assert2 (nil, cur:fetch())
-	assert2 (true, cur:close(), "couldn't close cursor")
+	assert2 (nil, cur:fetch{})
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	assert2 (false, cur:close())
 
 	-- retrieve data reusing the same table.
 	io.write ("reusing a table...")
 	cur = CUR_OK (CONN:execute ("select f1, f2, f3, f4 from t order by f1"))
-	local row, err = cur:fetch{}
+	local row, err = cur:fetch(fetch_table())
 	assert (type(row), "table", err)
 	assert2 ('a', row[1])
 	assert2 ('b', row[2])
@@ -222,14 +268,14 @@ function fetch_new_table ()
 	assert2 (nil, row.f2)
 	assert2 (nil, row.f3)
 	assert2 (nil, row.f4)
-	assert2 (nil, cur:fetch{})
-	assert2 (true, cur:close(), "couldn't close cursor")
+	assert2 (nil, cur:fetch(fetch_table()))
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	assert2 (false, cur:close())
 
 	-- retrieve data reusing the same table with alphabetic indexes.
 	io.write ("with alpha keys...")
 	cur = CUR_OK (CONN:execute ("select f1, f2, f3, f4 from t order by f1"))
-	local row, err = cur:fetch ({}, "a")
+	local row, err = cur:fetch (fetch_table(), "a")
 	assert (type(row), "table", err)
 	assert2 (nil, row[1])
 	assert2 (nil, row[2])
@@ -250,13 +296,13 @@ function fetch_new_table ()
 	assert2 ('h', row.f3)
 	assert2 ('i', row.f4)
 	assert2 (nil, cur:fetch(row, "a"))
-	assert2 (true, cur:close(), "couldn't close cursor")
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	assert2 (false, cur:close())
 
 	-- retrieve data reusing the same table with both indexes.
 	io.write ("with both keys...")
 	cur = CUR_OK (CONN:execute ("select f1, f2, f3, f4 from t order by f1"))
-	local row, err = cur:fetch ({}, "an")
+	local row, err = cur:fetch (fetch_table(), "an")
 	assert (type(row), "table", err)
 	assert2 ('a', row[1])
 	assert2 ('b', row[2])
@@ -277,7 +323,7 @@ function fetch_new_table ()
 	assert2 ('h', row.f3)
 	assert2 ('i', row.f4)
 	assert2 (nil, cur:fetch(row, "an"))
-	assert2 (true, cur:close(), "couldn't close cursor")
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	assert2 (false, cur:close())
 	-- clean the table.
 	assert2 (2, CONN:execute ("delete from t where f1 in ('a', 'f')"))
@@ -304,45 +350,41 @@ function fetch_many ()
 		assert2 ('v'..i, row[i])
 	end
 	assert2 (nil, cur:fetch (row))
-	assert2 (true, cur:close(), "couldn't close cursor")
-	assert2 (false, cur:close())
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	-- fetch values (with a table and default indexing).
 	io.write ("with a table...")
 	local cur = CUR_OK (CONN:execute ("select * from t where f1 = 'v1'"))
-	local row = cur:fetch {}
+	local row = cur:fetch(fetch_table())
 	assert2 ("string", type(row[1]), "error while trying to fetch many values (default indexing)")
 	for i = 1, TOTAL_FIELDS do
 		assert2 ('v'..i, row[i])
 	end
 	assert2 (nil, cur:fetch (row))
-	assert2 (true, cur:close(), "couldn't close cursor")
-	assert2 (false, cur:close())
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	-- fetch values (with numbered indexes on a table).
 	io.write ("with numbered keys...")
 	local cur = CUR_OK (CONN:execute ("select * from t where f1 = 'v1'"))
-	local row = cur:fetch ({}, "n")
+	local row = cur:fetch (fetch_table(), "n")
 	assert2 ("string", type(row[1]), "error while trying to fetch many values (numbered indexes)")
 	for i = 1, TOTAL_FIELDS do
 		assert2 ('v'..i, row[i])
 	end
 	assert2 (nil, cur:fetch (row))
-	assert2 (true, cur:close(), "couldn't close cursor")
-	assert2 (false, cur:close())
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	-- fetch values (with alphanumeric indexes on a table).
 	io.write ("with alpha keys...")
 	local cur = CUR_OK (CONN:execute ("select * from t where f1 = 'v1'"))
-	local row = cur:fetch ({}, "a")
+	local row = cur:fetch (fetch_table(), "a")
 	assert2 ("string", type(row.f1), "error while trying to fetch many values (alphanumeric indexes)")
 	for i = 1, TOTAL_FIELDS do
 		assert2 ('v'..i, row['f'..i])
 	end
 	assert2 (nil, cur:fetch (row))
-	assert2 (true, cur:close(), "couldn't close cursor")
-	assert2 (false, cur:close())
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	-- fetch values (with both indexes on a table).
 	io.write ("with both keys...")
 	local cur = CUR_OK (CONN:execute ("select * from t where f1 = 'v1'"))
-	local row = cur:fetch ({}, "na")
+	local row = cur:fetch (fetch_table(), "na")
 	assert2 ("string", type(row[1]), "error while trying to fetch many values (both indexes)")
 	assert2 ("string", type(row.f1), "error while trying to fetch many values (both indexes)")
 	for i = 1, TOTAL_FIELDS do
@@ -350,8 +392,7 @@ function fetch_many ()
 		assert2 ('v'..i, row['f'..i])
 	end
 	assert2 (nil, cur:fetch (row))
-	assert2 (true, cur:close(), "couldn't close cursor")
-	assert2 (false, cur:close())
+	assert2 (false, cur:close(), MSG_CURSOR_NOT_CLOSED)
 	-- clean the table.
 	assert2 (1, CONN:execute ("delete from t where f1 = 'v1'"))
 end
@@ -433,14 +474,19 @@ function column_info ()
 	assert2 (4, table.getn(names), "incorrect column names table")
 	assert2 (4, table.getn(types), "incorrect column types table")
 	for i = 1, table.getn(names) do
-		assert2 ("f"..i, names[i], "incorrect column names table")
+		assert2 ("f"..i, string.lower(names[i]), "incorrect column names table")
 		local type_i = types[i]
 		assert (type_i == QUERYING_STRING_TYPE_NAME, "incorrect column types table")
 	end
 	-- check if the tables are being reused.
 	local n2, t2 = cur:getcolnames(), cur:getcoltypes()
-	assert2 (names, n2, "getcolnames is rebuilding the table")
-	assert2 (types, t2, "getcoltypes is rebuilding the table")
+	if CHECK_GETCOL_INFO_TABLES then 
+		assert2 (names, n2, "getcolnames is rebuilding the table")
+		assert2 (types, t2, "getcoltypes is rebuilding the table")
+	else
+		assert2 (true, table_compare(names, n2), "getcolnames is inconsistent")
+		assert2 (true, table_compare(types, t2), "getcoltypes is inconsistent")
+	end
 	assert2 (true, cur:close(), "couldn't close cursor")
 	assert2 (false, cur:close())
 	-- clean the table.
@@ -451,7 +497,9 @@ end
 -- Escaping strings
 ---------------------------------------------------------------------
 function escape ()
-	assert2 ("a''b''c''d", CONN:escape"a'b'c'd")
+        local escaped = CONN:escape"a'b'c'd"
+	assert ("a\\'b\\'c\\'d" == escaped or "a''b''c''d" == escaped)
+	io.write (" escape")
 end
 
 ---------------------------------------------------------------------
@@ -567,13 +615,10 @@ if type(arg[1]) ~= "string" then
 end
 
 driver = arg[1]
-datasource = arg[2] or "luasql-test"
-username = arg[3] or nil
-password = arg[4] or nil
 
 -- Loading driver specific functions
 if arg[0] then
-	local path = string.gsub (arg[0], "^([^/]*%/).*$", "%1")
+	local path = string.gsub (arg[0], "^(.*%/)[^/]*$", "%1")
 	if path == "test.lua" then
 		path = ""
 	end
@@ -583,10 +628,15 @@ if arg[0] then
 		print ("LuaSQL test: couldn't find driver-specific test file ("..
 			file..").\nProceeding with general test")
 	else
+		print ("Loading driver-specific test file ("..file..").")
 		f ()
 	end
 end
 
+datasource = arg[2] or DEFAULT_TEST_DATABASE or "luasql-test"
+username = arg[3] or DEFAULT_USERNAME or nil
+password = arg[4] or DEFAULT_PASSWORD or nil
+
 -- Complete set of tests
 tests = {
 	{ "basic checking", basic_test },
@@ -596,20 +646,42 @@ tests = {
 	{ "fetch many", fetch_many },
 	{ "rollback", rollback },
 	{ "get column information", column_info },
-	{ "escape", escape },
 	{ "extensions", extensions_test },
 	{ "close objects", check_close },
 	{ "drop table", drop_table },
 	{ "close connection", close_conn },
 }
 
-require ("luasql."..driver)
-assert (luasql, "no luasql table")
+if string.find(_VERSION, " 5.0") then
+	luasql = assert(loadlib("./"..driver..".so", "luaopen_luasql_"..driver))()
+else
+	luasql = require ("luasql."..driver)
+end
+assert (luasql, "Could not load driver: no luasql table.")
+io.write (luasql._VERSION.." "..driver.." driver test.  "..luasql._COPYRIGHT.."\n")
 
 for i = 1, table.getn (tests) do
 	local t = tests[i]
 	io.write (t[1].." ...")
-	t[2] ()
+	local ok, err = xpcall (t[2], debug.traceback)
+	if not ok then
+		io.write ("\n"..err)
+		io.write"\n... trying to drop test table ..."
+		local ok, err = pcall (drop_table)
+		if not ok then
+			io.write (" failed: "..err)
+		else
+			io.write" OK !\n... and to close the connection ..."
+			local ok, err = pcall (close_conn)
+			if not ok then
+				io.write (" failed: "..err)
+			else
+				io.write" OK !"
+			end
+		end
+		io.write"\nThe test failed!\n"
+		return
+	end
 	io.write (" OK !\n")
 end
-
+io.write ("The test passed!\n")
